import { useRef } from 'react';
import { useIsomorphicLayoutEffect as useLayoutEffect } from '../../libs/isomorphic-layout-effect';
import * as ScrollLocker from '../../libs/scroll-locker';
/**
 * Raact hook that prevents content from scrolling on an element.
 *
 * @example
 * const Modal: FC<ModalProps> = (props) => {
 *   const { visible } = props;
 *   const containerRef = useRef(document.documentElement);
 *
 *   usePreventScroll({ enabled: visible, containerRef });
 *
 *   ...
 * }
 */
export function usePreventScroll(options) {
    var enabled = options.enabled, containerRef = options.containerRef;
    var elementRef = useRef(null);
    var lockedRef = useRef(false);
    useLayoutEffect(function () {
        var container = containerRef ? containerRef.current : null;
        if (elementRef.current === container) {
            return;
        }
        // We handle the case when the reference to the element has changed,
        // and the current one has a lock enabled
        if (enabled && lockedRef.current) {
            ScrollLocker.unlock(elementRef.current);
            ScrollLocker.lock(container);
        }
        elementRef.current = container;
    });
    useLayoutEffect(function () {
        if (!enabled) {
            return;
        }
        lockedRef.current = true;
        ScrollLocker.lock(elementRef.current);
        return function () {
            lockedRef.current = false;
            ScrollLocker.unlock(elementRef.current);
        };
    }, [enabled]);
}
