import { ElementTreeWalker } from './ElementTreeWalker';
import { adjustedTabIndex } from './adjustedTabIndex';
import { isFocusable } from './isFocusable';
var FocusDirection;
(function (FocusDirection) {
    FocusDirection[FocusDirection["Backward"] = 0] = "Backward";
    FocusDirection[FocusDirection["Forward"] = 1] = "Forward";
})(FocusDirection || (FocusDirection = {}));
var FocusNavigation = /** @class */ (function () {
    function FocusNavigation(scope) {
        this.walker = new ElementTreeWalker(scope, function (el) {
            return scope !== el && scope.contains(el) && isFocusable(el);
        });
    }
    FocusNavigation.prototype.findElement = function (dir, predicate) {
        var isForward = dir === FocusDirection.Forward;
        for (; this.walker.current; isForward ? this.walker.next() : this.walker.previous()) {
            if (predicate(this.walker.current)) {
                return this.walker.current;
            }
        }
        return null;
    };
    FocusNavigation.prototype.findElementWithTabIndex = function (tabIndex, dir) {
        return this.findElement(dir, function (el) { return adjustedTabIndex(el) === tabIndex; });
    };
    FocusNavigation.prototype.findFirstElementWithGreaterTabIndex = function (dir) {
        return this.findElement(dir, function (el) { return adjustedTabIndex(el) >= 0; });
    };
    FocusNavigation.prototype.findElementWithGreaterTabIndex = function (tabIndex) {
        var candidateTabIndex = Number.MAX_SAFE_INTEGER;
        var candidate = null;
        for (; this.walker.current; this.walker.next()) {
            var currentTabIndex = adjustedTabIndex(this.walker.current);
            if (currentTabIndex > tabIndex && (!candidate || currentTabIndex < candidateTabIndex)) {
                candidate = this.walker.current;
                candidateTabIndex = currentTabIndex;
            }
            // NOTE: the smallest tabIndex was found, but it is larger than the input tabIndex
            if (candidateTabIndex === tabIndex + 1) {
                break;
            }
        }
        this.walker.current = candidate;
        return candidate;
    };
    FocusNavigation.prototype.findElementWithLowerTabIndex = function (tabIndex) {
        var candidateTabIndex = 0;
        var candidate = null;
        for (; this.walker.current; this.walker.previous()) {
            var currentTabIndex = adjustedTabIndex(this.walker.current);
            if (currentTabIndex < tabIndex && currentTabIndex > candidateTabIndex) {
                candidate = this.walker.current;
                candidateTabIndex = currentTabIndex;
            }
            // NOTE: the largest tabIndex was found, but it is smaller than the input tabIndex
            if (candidateTabIndex === tabIndex - 1) {
                break;
            }
        }
        this.walker.current = candidate;
        return candidate;
    };
    FocusNavigation.prototype.findNextFocusable = function (tabbable) {
        var current = this.walker.current;
        var tabIndex = current ? adjustedTabIndex(current) : 0;
        if (current) {
            if (!tabbable) {
                return this.walker.next();
            }
            if (tabIndex >= 0) {
                this.walker.next();
                var candidate_1 = this.findElementWithTabIndex(tabIndex, FocusDirection.Forward);
                if (candidate_1) {
                    return candidate_1;
                }
            }
            else {
                var candidate_2 = this.findFirstElementWithGreaterTabIndex(FocusDirection.Forward);
                if (candidate_2) {
                    return candidate_2;
                }
            }
            if (tabIndex === 0) {
                return null;
            }
        }
        this.walker.first();
        if (!tabbable) {
            return this.walker.current;
        }
        var candidate = this.findElementWithGreaterTabIndex(tabIndex);
        if (candidate) {
            return candidate;
        }
        this.walker.first();
        return this.findElementWithTabIndex(0, FocusDirection.Forward);
    };
    FocusNavigation.prototype.findPreviousFocusable = function (tabbable) {
        var current = this.walker.current;
        if (!current) {
            this.walker.last();
        }
        else {
            this.walker.previous();
        }
        if (!tabbable) {
            return this.walker.current;
        }
        var tabIndex = current ? adjustedTabIndex(current) : 0;
        if (tabIndex < 0) {
            var candidate = this.findFirstElementWithGreaterTabIndex(FocusDirection.Backward);
            if (candidate) {
                return candidate;
            }
        }
        else {
            var candidate = this.findElementWithTabIndex(tabIndex, FocusDirection.Backward);
            if (candidate) {
                return candidate;
            }
        }
        this.walker.last();
        return this.findElementWithLowerTabIndex(tabIndex > 0 ? tabIndex : Number.MAX_SAFE_INTEGER);
    };
    Object.defineProperty(FocusNavigation.prototype, "scope", {
        get: function () {
            return this.walker.root;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FocusNavigation.prototype, "current", {
        get: function () {
            return this.walker.current;
        },
        enumerable: false,
        configurable: true
    });
    FocusNavigation.prototype.next = function (options) {
        if (options === void 0) { options = {}; }
        var tabbable = options.tabbable, from = options.from, wrap = options.wrap;
        if (from !== undefined) {
            this.walker.current = from;
        }
        var node = this.findNextFocusable(tabbable);
        if (!node && wrap) {
            return this.findNextFocusable(tabbable);
        }
        return node;
    };
    FocusNavigation.prototype.previous = function (options) {
        if (options === void 0) { options = {}; }
        var tabbable = options.tabbable, from = options.from, wrap = options.wrap;
        if (from !== undefined) {
            this.walker.current = from;
        }
        var node = this.findPreviousFocusable(tabbable);
        if (!node && wrap) {
            return this.findPreviousFocusable(tabbable);
        }
        return node;
    };
    FocusNavigation.prototype.first = function (tabbable) {
        return this.next({ from: null, tabbable: tabbable });
    };
    FocusNavigation.prototype.last = function (tabbable) {
        return this.previous({ from: null, tabbable: tabbable });
    };
    return FocusNavigation;
}());
export { FocusNavigation };
