"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FocusTrapManager = void 0;
var dom_utils_1 = require("../dom-utils");
var utils_1 = require("./utils");
var tabbable_1 = require("./utils/tabbable");
var Direction;
(function (Direction) {
    Direction[Direction["None"] = 0] = "None";
    Direction[Direction["Forward"] = 1] = "Forward";
    Direction[Direction["Backward"] = -1] = "Backward";
})(Direction || (Direction = {}));
var FocusTrapManager = /** @class */ (function () {
    function FocusTrapManager() {
        this.instances = [];
        this.direction = Direction.None;
        this.lastActiveElement = null;
        this.onDocumentFocusIn = this.onDocumentFocusIn.bind(this);
        this.onDocumentKeyDown = this.onDocumentKeyDown.bind(this);
        this.onMouseDown = this.onMouseDown.bind(this);
    }
    FocusTrapManager.getInstance = function () {
        if (!this.manager) {
            this.manager = new FocusTrapManager();
        }
        return this.manager;
    };
    FocusTrapManager.prototype.activate = function (instance) {
        this.preventLastFocusIn();
        if (this.instances.length === 0) {
            this.attachEvents();
        }
        this.instances.push(instance);
        this.autoFocus(instance);
    };
    FocusTrapManager.prototype.deactivate = function (instance) {
        this.preventLastFocusIn();
        this.instances = this.instances.filter(function (trap) { return trap !== instance; });
        if (this.instances.length === 0) {
            this.detachEvents();
        }
    };
    FocusTrapManager.prototype.autoFocus = function (instance) {
        var scope = instance.getScope();
        var autoFocus = instance.getOptions().autoFocus;
        var activeElement = (0, utils_1.getActiveElement)();
        if (autoFocus && (!activeElement || !(0, utils_1.isElementInScope)(scope, activeElement))) {
            (0, utils_1.focusFirstIn)(scope);
            // Set focus for modal container when not found focusable element inside.
            this.focusScope(scope);
        }
    };
    FocusTrapManager.prototype.focusScope = function (scope) {
        var activeElement = (0, utils_1.getActiveElement)();
        if (!activeElement || !(0, utils_1.isElementInScope)(scope, activeElement)) {
            (0, dom_utils_1.focusElement)(scope, true);
        }
    };
    FocusTrapManager.prototype.checkFocusInCurrentScope = function () {
        var instance = this.instances[this.instances.length - 1];
        if (instance) {
            this.checkFocusIn(instance);
        }
    };
    FocusTrapManager.prototype.preventLastFocusIn = function () {
        if (this.raf) {
            cancelAnimationFrame(this.raf);
        }
    };
    FocusTrapManager.prototype.checkFocusIn = function (instance) {
        var scope = instance.getScope();
        var _a = this, direction = _a.direction, lastActiveElement = _a.lastActiveElement;
        var activeElement = (0, utils_1.getActiveElement)();
        this.lastActiveElement = activeElement;
        this.direction = Direction.None;
        if (!activeElement || (0, utils_1.isFocusFree)() || (0, utils_1.isElementInScope)(scope, activeElement)) {
            return;
        }
        var tabbables = (0, tabbable_1.getTabbables)(scope, function (node) { return !(0, utils_1.isGuard)(node); });
        var activeIndex = tabbables.indexOf(activeElement);
        if (tabbables.length === 0) {
            this.focusScope(scope);
        }
        else if (activeIndex === -1) {
            var currentIndex = lastActiveElement ? tabbables.indexOf(lastActiveElement) : -1;
            var index = (0, utils_1.getNextTabbableIndex)(currentIndex, tabbables.length, direction);
            (0, dom_utils_1.focusElement)(tabbables[index]);
        }
    };
    FocusTrapManager.prototype.attachEvents = function () {
        document.addEventListener('focusin', this.onDocumentFocusIn, true);
        document.addEventListener('keydown', this.onDocumentKeyDown, true);
        document.addEventListener('mousedown', this.onMouseDown, true);
    };
    FocusTrapManager.prototype.detachEvents = function () {
        document.removeEventListener('focusin', this.onDocumentFocusIn, true);
        document.removeEventListener('keydown', this.onDocumentKeyDown, true);
        document.removeEventListener('mousedown', this.onMouseDown, true);
    };
    FocusTrapManager.prototype.onDocumentFocusIn = function () {
        var _this = this;
        // Use requestAnimationFrame for avoid problems with focus when activates new focus trap.
        this.raf = requestAnimationFrame(function () {
            _this.checkFocusInCurrentScope();
        });
    };
    FocusTrapManager.prototype.onDocumentKeyDown = function (event) {
        if (event.key !== 'Tab' || event.altKey || event.ctrlKey || event.metaKey) {
            return;
        }
        this.direction = event.shiftKey ? Direction.Backward : Direction.Forward;
    };
    FocusTrapManager.prototype.onMouseDown = function () {
        this.direction = Direction.None;
    };
    return FocusTrapManager;
}());
exports.FocusTrapManager = FocusTrapManager;
