"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAriaValueAttributes = exports.parseSegmentValue = exports.stepValue = void 0;
// Converts unicode number strings to real JS numbers.
// Numbers can be displayed and typed in many number systems, but JS
// only understands latin numbers.
// See https://www.fileformat.info/info/unicode/category/Nd/list.htm
// for a list of unicode numeric characters.
// Currently only Arabic and Latin numbers are supported, but more
// could be added here in the future.
// Keep this in sync with `isNumeric` below.
function parseNumber(str) {
    str = str
        // Arabic Indic
        .replace(/[\u0660-\u0669]/g, function (c) { return String(c.charCodeAt(0) - 0x0660); })
        // Extended Arabic Indic
        .replace(/[\u06f0-\u06f9]/g, function (c) { return String(c.charCodeAt(0) - 0x06f0); });
    return Number(str);
}
// Checks whether a unicode string could be converted to a number.
// Keep this in sync with `parseNumber` above.
function isNumeric(str) {
    return /^[0-9\u0660-\u0669\u06f0-\u06f9]+$/.test(str);
}
function stepValue(value, step, min, max, round) {
    if (round === void 0) { round = false; }
    var newValue = value;
    if (round) {
        newValue += step > 0 ? 1 : -1;
        var div = Math.abs(step);
        var mathRound = step > 0 ? Math.ceil : Math.floor;
        newValue = mathRound(newValue / div) * div;
    }
    else {
        newValue += step;
        if (newValue < min) {
            newValue = max - (min - newValue - 1);
        }
        else if (newValue > max) {
            newValue = min + (newValue - max - 1);
        }
    }
    if (newValue < min || newValue > max) {
        return step > 0 ? min : max;
    }
    return newValue;
}
exports.stepValue = stepValue;
function parseSegmentValue(type, str) {
    if (isNumeric(str) || /^[ap]$/i.test(str)) {
        var numberValue = parseNumber(str);
        if (type === 'dayPeriod') {
            var lower = str.toLowerCase();
            if (lower === 'a' || numberValue === 1) {
                return 1;
            }
            if (lower === 'p' || numberValue === 2) {
                return 2;
            }
            return null;
        }
        if (isFinite(numberValue)) {
            return numberValue;
        }
    }
    return null;
}
exports.parseSegmentValue = parseSegmentValue;
function getAriaValueAttributes(segment, options) {
    var value = segment.value, min = segment.min, max = segment.max;
    var attrs = { value: value !== null && value !== void 0 ? value : undefined, min: min, max: max };
    if (segment.type !== 'hour') {
        return attrs;
    }
    var hourCycle = options.hourCycle;
    var hours = value !== null && value !== void 0 ? value : 0;
    switch (hourCycle) {
        case 'h11':
            attrs.value = hours % 12;
            attrs.min = 0;
            attrs.max = 11;
            if (max < 12) {
                attrs.min = min;
                attrs.max = max;
            }
            else if (min >= 12) {
                attrs.min = min - 12;
                attrs.max = max - 12;
            }
            break;
        case 'h12':
            attrs.value = hours % 12 || 12;
            attrs.min = 1;
            attrs.max = 12;
            if (max < 12) {
                attrs.min = min;
                attrs.max = max;
            }
            else if (min >= 12) {
                attrs.min = min - 12;
                attrs.max = max - 12;
            }
            attrs.min = attrs.min || 12;
            attrs.max = attrs.max || 12;
            if (attrs.min > attrs.max) {
                attrs.min = 1;
                attrs.max = 12;
            }
            break;
        case 'h24':
            attrs.value = hours || 24;
            attrs.min = min || 24;
            attrs.max = max || 24;
            if (attrs.min > attrs.max) {
                attrs.min = 1;
                attrs.max = 24;
            }
            break;
        case 'h23':
            attrs.value = hours;
            attrs.min = min;
            attrs.max = max;
            break;
        default:
            throw new Error('Invalid hour cycle');
    }
    return attrs;
}
exports.getAriaValueAttributes = getAriaValueAttributes;
